import * as plugins from '../../../plugins.js';
import type { IRouteConfig } from '../../smart-proxy/models/route-types.js';
import type { IConnectionRecord } from '../../smart-proxy/models/interfaces.js';
import type { ILogger } from '../models/types.js';
import { createLogger } from '../models/types.js';
import { HttpStatus, getStatusText } from '../models/http-types.js';

export interface IRedirectHandlerContext {
  connectionId: string;
  connectionManager: any; // Avoid circular deps
  settings: any;
  logger?: ILogger;
}

/**
 * Handles HTTP redirect routes
 */
export class RedirectHandler {
  /**
   * Handle redirect routes
   */
  public static async handleRedirect(
    socket: plugins.net.Socket,
    route: IRouteConfig,
    context: IRedirectHandlerContext
  ): Promise<void> {
    const { connectionId, connectionManager, settings } = context;
    const logger = context.logger || createLogger(settings.logLevel || 'info');
    const action = route.action;

    // We should have a redirect configuration
    if (!action.redirect) {
      logger.error(`[${connectionId}] Redirect action missing redirect configuration`);
      socket.end();
      connectionManager.cleanupConnection({ id: connectionId }, 'missing_redirect');
      return;
    }

    // For TLS connections, we can't do redirects at the TCP level
    // This check should be done before calling this handler
    
    // Wait for the first HTTP request to perform the redirect
    const dataListeners: ((chunk: Buffer) => void)[] = [];

    const httpDataHandler = (chunk: Buffer) => {
      // Remove all data listeners to avoid duplicated processing
      for (const listener of dataListeners) {
        socket.removeListener('data', listener);
      }

      // Parse HTTP request to get path
      try {
        const headersEnd = chunk.indexOf('\r\n\r\n');
        if (headersEnd === -1) {
          // Not a complete HTTP request, need more data
          socket.once('data', httpDataHandler);
          dataListeners.push(httpDataHandler);
          return;
        }

        const httpHeaders = chunk.slice(0, headersEnd).toString();
        const requestLine = httpHeaders.split('\r\n')[0];
        const [method, path] = requestLine.split(' ');

        // Extract Host header
        const hostMatch = httpHeaders.match(/Host: (.+?)(\r\n|\r|\n|$)/i);
        const host = hostMatch ? hostMatch[1].trim() : '';

        // Process the redirect URL with template variables
        let redirectUrl = action.redirect.to;
        redirectUrl = redirectUrl.replace(/\{domain\}/g, host);
        redirectUrl = redirectUrl.replace(/\{path\}/g, path || '');
        redirectUrl = redirectUrl.replace(/\{port\}/g, socket.localPort?.toString() || '80');

        // Prepare the HTTP redirect response
        const redirectResponse = [
          `HTTP/1.1 ${action.redirect.status} Moved`,
          `Location: ${redirectUrl}`,
          'Connection: close',
          'Content-Length: 0',
          '',
          '',
        ].join('\r\n');

        if (settings.enableDetailedLogging) {
          logger.info(
            `[${connectionId}] Redirecting to ${redirectUrl} with status ${action.redirect.status}`
          );
        }

        // Send the redirect response
        socket.end(redirectResponse);
        connectionManager.initiateCleanupOnce({ id: connectionId }, 'redirect_complete');
      } catch (err) {
        logger.error(`[${connectionId}] Error processing HTTP redirect: ${err}`);
        socket.end();
        connectionManager.initiateCleanupOnce({ id: connectionId }, 'redirect_error');
      }
    };

    // Setup the HTTP data handler
    socket.once('data', httpDataHandler);
    dataListeners.push(httpDataHandler);
  }
}